--*****************************************************************************
--*  Copyright (c) 2015 by Michael Fischer (www.emb4fun.de)
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  24.05.2015  mifi  First version, based on my wb_gpio component.
--*  07.11.2015  mifi  Tested with Wishbone and Avalon Master.
--*****************************************************************************

--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity av_gpio_if is
	port (
		clk           : in  std_logic                     := '0';             --   clock.clk
		reset         : in  std_logic                     := '0';             --   reset.reset
      
		avs_address   : in  std_logic_vector(2 downto 0)  := (others => '0'); --      s0.address
		avs_read      : in  std_logic                     := '0';             --        .read
		avs_readdata  : out std_logic_vector(15 downto 0);                    --        .readdata
		avs_write     : in  std_logic                     := '0';             --        .write
		avs_writedata : in  std_logic_vector(15 downto 0) := (others => '0'); --        .writedata
      
		port_o        : out std_logic_vector(15 downto 0);                    -- conduit.export
		port_i        : in  std_logic_vector(15 downto 0) := (others => '0')  --        .export
	);
end entity av_gpio_if;

--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of av_gpio_if is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------

   component gpio_reg is
	   port (
		   clk_i       : in  std_logic                     := '0';             --   clock.clk
		   reset_i     : in  std_logic                     := '0';             --   reset.reset
      
		   address_i   : in  std_logic_vector(2 downto 0)  := (others => '0'); --      s0.address
		   read_i      : in  std_logic                     := '0';             --        .read
		   readdata_o  : out std_logic_vector(15 downto 0);                    --        .readdata
		   write_i     : in  std_logic                     := '0';             --        .write
		   writedata_i : in  std_logic_vector(15 downto 0) := (others => '0'); --        .writedata
      
         wb_ack_o    : out std_logic;
      
		   port_o      : out std_logic_vector(15 downto 0);                    -- conduit.export
		   port_i      : in  std_logic_vector(15 downto 0) := (others => '0')  --        .export
	   );
   end component gpio_reg;   
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------
   
begin

   inst_gpio : gpio_reg 
      port map ( 
         clk_i       => clk,
         reset_i     => reset,

		   address_i   => avs_address,
		   read_i      => avs_read,
		   readdata_o  => avs_readdata,
		   write_i     => avs_write,
		   writedata_i => avs_writedata,
         
         wb_ack_o    => open,
      
		   port_o      => port_o,
		   port_i      => port_i
      );

end architecture syn;

-- *** EOF ***
