--*****************************************************************************
--*  Copyright (c) 2014-2015 by Michael Fischer (www.emb4fun.de) 
--*  All rights reserved.
--*
--*  Redistribution and use in source and binary forms, with or without 
--*  modification, are permitted provided that the following conditions 
--*  are met:
--*  
--*  1. Redistributions of source code must retain the above copyright 
--*     notice, this list of conditions and the following disclaimer.
--*  2. Redistributions in binary form must reproduce the above copyright
--*     notice, this list of conditions and the following disclaimer in the 
--*     documentation and/or other materials provided with the distribution.
--*  3. Neither the name of the author nor the names of its contributors may 
--*     be used to endorse or promote products derived from this software 
--*     without specific prior written permission.
--*
--*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
--*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
--*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
--*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
--*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
--*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
--*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
--*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
--*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
--*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
--*  SUCH DAMAGE.
--*
--*****************************************************************************
--*  History:
--*
--*  19.06.2014  mifi  First version.
--*****************************************************************************

--*****************************************************************************
--*  DEFINE: Library                                                          *
--*****************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


--*****************************************************************************
--*  DEFINE: Entity                                                           *
--*****************************************************************************

entity wb_syscon is
   port (
      -- None Wishbone
      clk_i    : in  std_logic := '0';
      reset_i  : in  std_logic := '0';
      clk_10_o : out std_logic;

      -- Wishbone
      clk_o    : out std_logic;
      rst_o    : out std_logic
   );
end entity wb_syscon;


--*****************************************************************************
--*  DEFINE: Architecture                                                     *
--*****************************************************************************

architecture syn of wb_syscon is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------

   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   component pll_sys
     port ( 
        inclk0  : in  std_logic := '0';
        c0      : out std_logic;
        c1      : out std_logic;
        locked  : out std_logic 
     );
   end component pll_sys;
   

   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------

   signal clk_sys     : std_logic := '0';
   signal pll_locked  : std_logic := '0';
   signal reset       : std_logic := '0';
   signal reset_s1    : std_logic := '1';
   signal reset_s2    : std_logic := '1';
   signal reset_s3    : std_logic := '1';
   signal reset_sync  : std_logic := '1';
   
begin

   -- PLL
   inst_pll_sys : pll_sys
      port map ( 
         inclk0 => clk_i,
         c0     => clk_sys,
         c1     => clk_10_o,
         locked => pll_locked
      );

   --
   -- In general it is a bad idea to use an asynchhronous Reset signal.
   -- But it is only a bad idea in case of asynchhronous deasserting. 
   -- Therefore the deasserting of the Reset signal must be synchronized.
   --               
               
   -- Asynchronous assert  
   reset <= '1' when ((reset_i = '1') OR (pll_locked = '0')) else '0';

   -- Synchronize deassert
   process (clk_sys, reset)
   begin
      if (reset = '1') then
         reset_s1 <= '1';
         reset_s2 <= '1';
         reset_s3 <= '1';
      elsif rising_edge(clk_sys) then
         reset_s1 <= '0';
         reset_s2 <= reset_s1;
         reset_s3 <= reset_s2; 
      end if;
   end process;   

   -- The deassert edge is now synchronized   
   reset_sync <= reset_s3; 
               

   --------------------------------------------------------
   -- Output
   --------------------------------------------------------
   clk_o <= clk_sys;
   rst_o <= reset_sync;               

end architecture syn;

-- *** EOF ***

